'Copyright (c) Microsoft Corporation.  All rights reserved

Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports System.IO ' required for file i/o 
Imports VBSideBarGadget.PortableDeviceConstants 'Namespace containing defns for WPD constants
Imports System.Runtime.InteropServices 'required for WPD prop retrieval 

Public Class Form1
    ' 
    ' Get an instance of the device manager 
    ' 
    Dim devMgr As New PortableDeviceApiLib.PortableDeviceManagerClass()

    ' Create our client information collection 
    Dim pValues As PortableDeviceApiLib.IPortableDeviceValues = DirectCast(New PortableDeviceTypesLib.PortableDeviceValuesClass(), PortableDeviceApiLib.IPortableDeviceValues)

    ' Create a new IPortableDevice instance 
    Dim pPortableDevice As New PortableDeviceApiLib.PortableDeviceClass()

    Dim strVal As String = [String].Empty
    Dim iVal As Integer = 0

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim i As UInteger = 0
        Dim cDevices As UInteger = 0
        Dim strDeviceID As String = [String].Empty
        Dim strFriendlyName As String = [String].Empty

        Try
            ' Retrieve a count of connected WPD devices. 
            devMgr.GetDevices(Nothing, cDevices)

            ' Iterate through the connected WPD devices--searching for 
            ' a TemperatureSensor device. If it's found, open the device 
            ' and begin retrieving the temperature property. 
            If cDevices > 0 Then
                ' Retrieve the PnP identifiers for each 
                ' connected device. 
                Dim strPnPDeviceIDs As String() = New String(cDevices - 1) {}
                devMgr.GetDevices(strPnPDeviceIDs, cDevices)
                For i = 0 To cDevices - 1

                    ' For each connected device, retrieve the friendly 
                    ' name and compare it to the TempSensor's name. 
                    strFriendlyName = RetrieveFriendlyName(devMgr, strPnPDeviceIDs(i))
                    If strFriendlyName = "Parallax BS2 Temperature Sensor" Then
                        ' Open the device and begin retrieving the temperature 
                        StartProcessing(pPortableDevice, pValues, strPnPDeviceIDs(i))
                    End If
                Next
            End If

        Catch exp As Exception
            MessageBox.Show(exp.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Stop)
        End Try
    End Sub

    ''' <summary> 
    ''' Returns the friendly-name for a given device. 
    ''' </summary> 
    ''' <param name="PortableDeviceManager">Interface used to retrieve friendly name</param> 
    ''' <param name="PnPDeviceID">Plug n Play identifier for given device.</param> 
    ''' <returns>String specifying device's friendly name.</returns> 
    Private Function RetrieveFriendlyName(ByVal PortableDeviceManager As PortableDeviceApiLib.PortableDeviceManagerClass, ByVal PnPDeviceID As String) As String
        Dim cFriendlyName As UInteger = 0
        Dim usFriendlyName As UShort()
        Dim strFriendlyName As String = [String].Empty

        Try
            ' First, pass NULL as the LPWSTR return string parameter to get the total number 
            ' of characters to allocate for the string value. 
            PortableDeviceManager.GetDeviceFriendlyName(PnPDeviceID, Nothing, cFriendlyName)

            ' Second allocate the number of characters needed and retrieve the string value. 

            usFriendlyName = New UShort(cFriendlyName - 1) {}
            If usFriendlyName.Length > 0 Then
                PortableDeviceManager.GetDeviceFriendlyName(PnPDeviceID, usFriendlyName, cFriendlyName)

                ' We need to convert the array of ushorts to a string, one 
                ' character at a time. 
                For Each letter As UShort In usFriendlyName
                    If letter <> 0 Then
                        strFriendlyName += Microsoft.VisualBasic.ChrW(letter)
                    End If
                Next

                ' Return the friendly name 
                Return strFriendlyName
            Else
                Return Nothing
            End If

        Catch exp As Exception
            MessageBox.Show(exp.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Stop)
            Return Nothing
        End Try
    End Function

    ''' <summary> 
    ''' Establishes a connection with the temperature-sensor device and 
    ''' enables the timer-event on 3-second intervals. (Each time this 
    ''' event fires, the application retrieves the current temperature.) 
    ''' </summary> 
    ''' <param name="pPortableDevice">Interface used to acces the device.</param> 
    ''' <param name="pValues">Interface used to specify client information for device.</param> 
    ''' <param name="PnPDeviceID">Plug n Play device identifier.</param> 
    Private Sub StartProcessing(ByVal pPortableDevice As PortableDeviceApiLib.PortableDeviceClass, ByVal pValues As PortableDeviceApiLib.IPortableDeviceValues, ByVal PnPDeviceID As String)
        Try
            ' 
            ' Open the temp-sensor device 
            ' 
            ' We have to provide at the least our name, version, revision 
            pValues.SetStringValue(PortableDevicePKeys.WPD_CLIENT_NAME, "Temp-Sensor Client")
            pValues.SetUnsignedIntegerValue(PortableDevicePKeys.WPD_CLIENT_MAJOR_VERSION, 1)
            pValues.SetUnsignedIntegerValue(PortableDevicePKeys.WPD_CLIENT_MINOR_VERSION, 0)
            pValues.SetUnsignedIntegerValue(PortableDevicePKeys.WPD_CLIENT_REVISION, 2)

            ' We are now ready to open a connection to the device 
            ' We'll assume deviceID contains a valid WPD device path and connect to it 
            pPortableDevice.Open(PnPDeviceID, pValues)

            ' Retrieve and display the initial temperature 
            strVal = RetrieveTemp(pPortableDevice, "Temperature")
            iVal = Convert.ToInt32(strVal)

            ' Enable the timer for UI 
            Timer1.Interval = 3000
            ' 3 second intervals 
            Timer1.Enabled = True

        Catch exp As Exception
            MessageBox.Show(exp.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Stop)
        End Try

    End Sub

    ''' <summary> 
    ''' Retrieves the current temperature from a connected 
    ''' temperature-sensor device. 
    ''' </summary> 
    ''' <param name="pPortableDevice">Interface used to access temp-sensor device.</param> 
    ''' <param name="objectID">Object ID used when retrieving temperature.</param> 
    ''' <returns>Temperature (Kelvin) as a string</returns> 
    Private Function RetrieveTemp(ByVal pPortableDevice As PortableDeviceApiLib.PortableDeviceClass, ByVal objectID As String) As String
        Dim strTemp As String = [String].Empty

        Try
            ' Receives current temp 
            ' 
            ' Retrieve IPortableDeviceContent interface required to 
            ' obtain the IPortableDeviceProperties interface 
            ' 
            Dim pContent As PortableDeviceApiLib.IPortableDeviceContent
            pContent = Nothing
            pPortableDevice.Content(pContent)

            ' 
            ' Retrieve IPortableDeviceProperties interface required 
            ' to get all the properties 
            ' 
            Dim pProperties As PortableDeviceApiLib.IPortableDeviceProperties
            pProperties = Nothing
            pContent.Properties(pProperties)

            ' 
            ' Create a key collection 
            ' 
            Dim pKeyCollection As PortableDeviceApiLib.IPortableDeviceKeyCollection = DirectCast(New PortableDeviceTypesLib.PortableDeviceKeyCollectionClass(), PortableDeviceApiLib.IPortableDeviceKeyCollection)

            ' Add the TEMP_SENSOR_READING property to the key collection 
            ' This value is defined in PortableDeviceConstans.cs 
            pKeyCollection.Add(PortableDevicePKeys.TEMP_SENSOR_READING)

            ' 
            ' Call the GetValues to retrieve the prop values for the given 
            ' key. Then, retrieve the TEMP_SENSOR_CURRENT_VALUE property 
            ' by calling the GetStringValue method. 
            ' 
            Dim pPropValues As PortableDeviceApiLib.IPortableDeviceValues
            pPropValues = Nothing
            pProperties.GetValues(objectID, pKeyCollection, pPropValues)
            Dim iTemp As Integer = 0
            pPropValues.GetSignedIntegerValue(PortableDevicePKeys.TEMP_SENSOR_READING, iTemp)

            If iTemp > 0 Then
                Return iTemp.ToString()
            Else
                Return "Invalid temperature!"
            End If

        Catch exp As Exception
            MessageBox.Show(exp.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Stop)
            Return "Invalid temperature!"
        End Try

    End Function


    ''' <summary> 
    ''' When the timer-tick event is fired, we retrieve the current temperature 
    ''' from the connected device and write it to an ASCII file and to the 
    ''' form's title bar. 
    ''' </summary> 
    ''' <param name="sender"></param> 
    ''' <param name="e"></param> 
    ''' 
    Private Sub Timer1_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles Timer1.Tick
        Dim dtNow As DateTime = DateTime.Now ' Use DateTime structure to retrieve timestamp 
        Dim tempEnvVar As String 'Receives path to %TEMP%

        Try
            ' Retrieve the temp prop 
            strVal = RetrieveTemp(pPortableDevice, "Temperature")
            iVal = KelvinToFarenheit(strVal)

            'Retrieve the path to %TEMP%
            tempEnvVar = Environment.GetEnvironmentVariable("TEMP")

            ' Use the StreamWriter object for file I/O 
            Dim sw As New StreamWriter(tempEnvVar & "\tempdata.txt")

            ' Write the data to the file 
            sw.WriteLine("<P>Office Temperature<P>" + Convert.ToString(iVal) + "&deg; Farenheit")

            ' Write the data to the textbox
            TextBox1.Text = Convert.ToString(iVal) + " Farenheit"

            ' Close the streamwriter object 
            sw.Close()

        Catch exp As Exception
            MessageBox.Show(exp.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Stop)
        End Try

    End Sub

    ''' <summary> 
    ''' Converts a temperature in degrees Kelvin to its Farenheit equivalent 
    ''' </summary> 
    ''' <param name="strTemp"></param> 
    ''' <returns>Temperature in degrees Kelvin</returns> 
    Private Function KelvinToFarenheit(ByVal strTemp As String) As Integer
        Dim dTemp As Double = 0
        Dim iVal As Integer = 0

        Try
            dTemp = Convert.ToDouble(strTemp)
            iVal = CInt((((dTemp - 273.15) * 1.8) + 32))
            Return iVal
        Catch exp As Exception
            MessageBox.Show(exp.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Stop)
        End Try
    End Function



End Class
